/*
 * config.h
 *
 *		75.663 TFG - Arduino - 2016-17 - Estudios de Informatica Multimedia y Telecomunicacion
 *
 *      Author: Miguel Angel Sanchez Munoz
 */

#ifndef CONFIG_H_
#define CONFIG_H_

#include <utility/EspDrv.h>

/*--------------------------------------------------
 *
 * Client information, ID, etc.
 *
 *--------------------------------------------------*/
const char* ID = "salon";


/*--------------------------------------------------
 *
 * Definitions for enable debug and serial port baudrate.
 *
 *--------------------------------------------------*/
//#define DEBUG
#define DEBUG_BAUD 115200

/*--------------------------------------------------
 *
 * Definitions for enable sensors/actuators.
 *
 *--------------------------------------------------*/
#define LDR
#define DHT
#define BUTTON
#define RELAY
#define MIC

/*--------------------------------------------------
 *
 * Data Queue type definition.
 *
 *--------------------------------------------------*/
typedef struct SensorData
{
	char* topic;
	double data;
} SensorData;

/*--------------------------------------------------
 *
 * Pinout definitions for hardware connection interface.
 *
 *--------------------------------------------------*/
#define RX 2 // SoftwareSerial RX
#define TX 3 // SoftwareSerial TX
#define BUTTON_PIN 4
#define DHT11_PIN 5
#define RELAY_PIN 6
#define MIC_PIN_D 7
#define LIGTH_PIN A0
#define MIC_PIN_A A1
#define LED_PIN 12

/*--------------------------------------------------
 *
 * Control variables of the actuator devices.
 *
 *--------------------------------------------------*/
bool RELAY_STATUS = false;
bool BUTTON_STATUS = true;
bool LED_STATUS = false;

/*--------------------------------------------------
 *
 * Configuration for network
 *
 *--------------------------------------------------*/
// Update these with values suitable for your network.
char ssid[] = "ArduSmartHome";            // your network SSID (name)
char pass[] = "ArduSmartHome";        // your network password
int status = WL_IDLE_STATUS;     // the Wifi radio's status
const char*  MQTT_SERVER = "192.168.240.1";
const int MQTT_PORT = 1883;
const char* MQTT_USER = "ardusmarthome";
const char* MQTT_PASS = "ardusmarthome";

/*--------------------------------------------------
 *
 * MQTT Topics
 *
 *--------------------------------------------------*/
const char* TopicLightData = "light/data";
const char* TopicTempData = "temperature/data";
const char* TopicHumiData = "humidity/data";
const char* TopicRelayStatus = "relay/status";
const char* TopicRelaySet = "relay/set";
const char* TopicButtonStatus = "button/status";
const char* TopicMicData = "mic/data";

#endif /* CONFIG_H_ */
