/*
 * main.cpp
 *
 *		75.663 TFG - Arduino - 2016-17 - Estudios de Informatica Multimedia y Telecomunicacion
 *
 *      Author: Miguel Angel Sanchez Munoz
 */
/* Application includes. */
#include "config.h"
#include <Arduino.h>
#include <TaskScheduler.h>
#include <WiFiEspClient.h>
#include <WiFiEsp.h>
#include <PubSubClient.h>
#include <QueueArray.h>
#include <dht.h>
#include <SoftwareSerial.h>

SoftwareSerial Serial1(RX, TX);
WiFiEspClient espClient;
PubSubClient client(espClient);
QueueArray <SensorData> queue;
Scheduler runner, hprunner;

char TopicIDLightData[30];
char TopicIDTempData[30];
char TopicIDHumiData[30];
char TopicIDRelayStatus[30];
char TopicIDRelaySet[30];
char TopicIDButtonSet[30];
char TopicIDButtonStatus[30];
char TopicIDMicData[30];


/* Task functions and callback methods prototypes. */

	bool tMQTTSetup();
	void tMQTTClient();
	void MQTTCallback(char* topic, byte* payload, unsigned int length);
	Task tMQTT(TASK_IMMEDIATE, TASK_FOREVER, &tMQTTClient, &hprunner, false, &tMQTTSetup);

	bool tWifiSetup();
	void tWifiCallback();
	Task tWifi(TASK_MINUTE, TASK_FOREVER, &tWifiCallback, &runner, false, &tWifiSetup);

#ifdef DHT
	bool tDHTSetup();
	void tDHTCallback();
	Task tDHT(TASK_SECOND * 5, TASK_FOREVER, &tDHTCallback, &runner, false, &tDHTSetup);
#endif
#ifdef LDR
	bool tLightSetup();
	void tLightCallback();
	Task tLight(TASK_SECOND * 2, TASK_FOREVER, &tLightCallback, &runner, false, &tLightSetup);
#endif
#ifdef BUTTON
	bool tButtonSetup();
	void tButtonCallback();
	Task tButton(TASK_IMMEDIATE, TASK_FOREVER, &tButtonCallback, &runner, false, &tButtonSetup);
#endif
#ifdef RELAY
	bool tRelaySetup();
	void tRelayCallback();
	Task tRelay(TASK_IMMEDIATE, TASK_FOREVER, &tRelayCallback, &runner, false, &tRelaySetup);
#endif
#ifdef MIC
	bool tMicSetup();
	void tMicCallback();
	Task tMic(TASK_SECOND * 2, TASK_FOREVER, &tMicCallback, &runner, false, &tMicSetup);
#endif


#ifdef DHT
/*--------------------------------------------------
 *@brief		Task Temperature/Humidity sersor fuctions. Captures
 *				ambient Temperature/Humidity level.
 *
 * @param[in]	None
 *
 * @return 		None
 *--------------------------------------------------*/
dht DHT11;
SensorData temperature;
SensorData humidity;
int DHTLoop = 1;
int samples = 6;
double tempAverage = 0;
double humiAverage = 0;

bool tDHTSetup() {

	temperature.topic = TopicIDTempData;
	humidity.topic = TopicIDHumiData;
	return true;
}

void tDHTCallback() {
#ifdef DEBUG
	Serial.println("tDHTCallback");
#endif

	DHT11.read11(DHT11_PIN);	// READ DATA

	tempAverage = tempAverage + DHT11.temperature;
	humiAverage = humiAverage + DHT11.humidity;

	if (DHTLoop == samples) {

		temperature.data = tempAverage / samples;
		tempAverage = 0;
		humidity.data = humiAverage / samples;
		humiAverage = 0;

		if (!queue.isFull()) {
			queue.enqueue(temperature);
		}

		if (!queue.isFull()) {
			queue.enqueue(humidity);
		}
		DHTLoop = 0;
	}

	DHTLoop ++;
}
/*--------------------------------------------------*/
#endif

#ifdef LDR
/*--------------------------------------------------
 *@brief		Task Light sersor fuctions. Captures ambient light level.
 *
 * @param[in]	None
 *
 * @return 		None
 *--------------------------------------------------*/
SensorData lightData;

bool tLightSetup() {

	lightData.topic = TopicIDLightData;

	return true;
}

void tLightCallback() {
#ifdef DEBUG
	Serial.println("tLightCallback");
#endif

	lightData.data = analogRead(LIGTH_PIN);

	if (!queue.isFull()) {
		queue.enqueue(lightData);
	}
}
/*--------------------------------------------------*/
#endif

#ifdef BUTTON
/*--------------------------------------------------
 *@brief		Task Button sersor fuctions. Human interacction.
 *
 * @param[in]	None
 *
 * @return 		None
 *--------------------------------------------------*/
SensorData buttonData;
bool BUTTON_OLD_STATUS = true;

bool tButtonSetup() {

	pinMode (BUTTON_PIN, INPUT);
	buttonData.topic = TopicIDButtonStatus;

	return true;
}

void tButtonCallback() {
#ifdef DEBUG
	Serial.println("tButtonCallback");
#endif

	BUTTON_STATUS = (bool)digitalRead(BUTTON_PIN);

	if(!BUTTON_STATUS){
		buttonData.data = !BUTTON_STATUS;
		if (!queue.isFull()) {
			queue.enqueue(buttonData);
		}
	}
}
/*--------------------------------------------------*/
#endif

#ifdef RELAY
/*--------------------------------------------------
 *@brief		Task Relay actuator fuctions.Allows to operate with
 *				large voltages, lamps or thermostats
 *
 * @param[in]	None
 *
 * @return 		None
 *--------------------------------------------------*/
SensorData relayData;
bool RELAY_OLD_STATUS = false;

bool tRelaySetup() {

	pinMode (RELAY_PIN, OUTPUT);
	relayData.topic = TopicIDRelayStatus;

	return true;
}

void tRelayCallback() {
#ifdef DEBUG
	Serial.println("tRelayCallback");
#endif

	digitalWrite(RELAY_PIN, RELAY_STATUS);

	if (RELAY_STATUS != RELAY_OLD_STATUS){
		RELAY_OLD_STATUS = RELAY_STATUS;
		relayData.data = RELAY_STATUS;
		if (!queue.isFull()) {
			queue.enqueue(relayData);
		}
	}
}
/*--------------------------------------------------*/
#endif

#ifdef MIC
/*--------------------------------------------------
 *@brief		Task Micro sersor fuctions. Captures ambient sound level.
 *
 * @param[in]	None
 *
 * @return 		None
 *--------------------------------------------------*/
SensorData micData;

bool tMicSetup() {

	pinMode (MIC_PIN_D, INPUT);
	micData.topic = TopicIDMicData;

	return true;
}

void tMicCallback() {
#ifdef DEBUG
	Serial.println("tMicCallback");
#endif

	micData.data = analogRead(MIC_PIN_A);

	if (!queue.isFull()) {
		queue.enqueue(micData);
	}
}
/*--------------------------------------------------*/
#endif


/*--------------------------------------------------
 *@brief		Task control network connexion fuctions. It makes the connection
 *				with the Wi-Fi network and controls periodically
 *
 * @param[in]	None
 *
 * @return 		None
 *--------------------------------------------------*/
bool tWifiSetup() {

	Serial1.begin(9600);
	WiFi.init(&Serial1);

	return true;
}

void tWifiCallback() {
#ifdef DEBUG
	Serial.println("tWifiCallback");
#endif

	if (WiFi.status() != WL_CONNECTED) {

		tMQTT.disable();

    	status = WiFi.begin(ssid, pass);

		if(status == WL_CONNECTED){
#ifdef DEBUG
			Serial.print("Wifi connected: ");
			Serial.println(WiFi.localIP());
#endif
			tMQTT.enable();
		}
    }
}
/*--------------------------------------------------*/

/*--------------------------------------------------
 *@brief		Task Debug fuctions. Print debugging information
 *				through the serial port.
 *
 * @param[in]	None
 *
 * @return 		None
 *--------------------------------------------------*/
SensorData data;
char msgPayload[8] = "";

void MQTTCallback(char* topic, byte* payload, unsigned int length) {
#ifdef DEBUG
	Serial.println("MQTTCallback");
#endif

  if (strcmp(topic, TopicIDRelaySet) == 0){
	if ( (char)payload[0] == '1') {
		RELAY_STATUS = true;
	} else {
		RELAY_STATUS = false;
	}
  }

}

bool tMQTTSetup() {

	pinMode (LED_PIN, OUTPUT);

	sprintf (TopicIDLightData, "%s/%s", ID, TopicLightData);
	sprintf (TopicIDTempData, "%s/%s", ID, TopicTempData);
	sprintf (TopicIDHumiData, "%s/%s", ID, TopicHumiData);
	sprintf (TopicIDRelaySet, "%s/%s", ID, TopicRelaySet);
	sprintf (TopicIDRelayStatus, "%s/%s", ID, TopicRelayStatus);
	sprintf (TopicIDButtonStatus, "%s/%s", ID, TopicButtonStatus);
	sprintf (TopicIDMicData, "%s/%s", ID, TopicMicData);

	client.setServer(MQTT_SERVER, MQTT_PORT);
	client.setCallback(MQTTCallback);

	return true;
}

void tMQTTClient() {
#ifdef DEBUG
	Serial.println("tMQTTClient");
#endif

	if (!client.connected()) {

		if (client.connect(ID, MQTT_USER, MQTT_PASS)) {
#ifdef DEBUG
			Serial.println("MQTT Client connect");
#endif
			client.publish(ID, "hello");

#ifdef RELAY
			client.subscribe(TopicIDRelaySet, 1);
#endif

		}
	} else {

		client.loop();

		if (!queue.isEmpty()) {
			data = queue.dequeue();
			dtostrf(data.data, 2, 2, msgPayload);
#ifdef DEBUG
			Serial.print(data.topic);
			Serial.println(msgPayload);
#else
			client.publish(data.topic, msgPayload);
#endif
		  }

	}

}
/*--------------------------------------------------*/


/*--------------------------------------------------
 *@brief		Main Setup application.
 *
 * @param[in]	None
 *
 * @return 		None
 *--------------------------------------------------*/
void setup() {
#ifdef DEBUG
	Serial.begin(DEBUG_BAUD);
#endif

	runner.init();

	runner.setHighPriorityScheduler(&hprunner);

	runner.addTask(tMQTT);

	runner.addTask(tWifi);
	tWifi.enable();

#ifdef LDR
	runner.addTask(tLight);
	tLight.enable();
#endif
#ifdef DHT
	runner.addTask(tDHT);
	tDHT.enable();
#endif
#ifdef BUTTON
	runner.addTask(tButton);
	tButton.enable();
#endif
#ifdef RELAY
	runner.addTask(tRelay);
	tRelay.enable();
#endif
#ifdef MIC
	runner.addTask(tMic);
	tMic.enable();
#endif

}
/*--------------------------------------------------*/

/*--------------------------------------------------
 *@brief		Main Loop application.
 *
 * @param[in]	None
 *
 * @return 		Function never end.
 *--------------------------------------------------*/
void loop() {
  runner.execute();
}
/*--------------------------------------------------*/
